@@ -44,6 +44,18 @@
 //! - The command-line client is very basic. If you need more features, use [yedb
 //!   Python CLI](https://github.com/alttch/yedb-py).
 //! 
+//! ## Additional features over YEDB specs
+//! 
+//! - "delete" command does not delete keys, they are moved to .trash folder
+//!   instead
+//! 
+//! - .trash folder is being cleaned when "purge" method is invoked.
+//! 
+//! - "auto\_bak" property tells server to automatically create backup key versions
+//!   when key data is modified
+//! 
+//! - "bak" keys are hidden
+//! 
 //! ## Client/server
 //! 
 //! Binaries available at the [releases
@@ -154,6 +166,7 @@ use fs2::FileExt;
 use glob::glob;
 use jsonschema::{Draft, JSONSchema};
 use lru::LruCache;
+use regex::Regex;
 use serde::{de::Error as deError, Deserialize, Deserializer, Serialize, Serializer};
 use serde_json::Value;
 use sha2::{Digest, Sha256};
@@ -166,14 +179,21 @@ use std::time::{Duration, Instant, SystemTime};
 
 use log::{debug, error, warn};
 
+#[macro_use]
+extern crate lazy_static;
+
 pub const SERVER_ID: &str = "yedb-altt-rs";
-pub const VERSION: &str = "0.0.6";
+pub const VERSION: &str = "0.0.7";
 pub const ENGINE_VERSION: u8 = 1;
 
 pub const DEFAULT_CACHE_SIZE: usize = 1000;
 
 const SLEEP_STEP: Duration = Duration::from_millis(50);
 
+lazy_static! {
+    static ref RE_BAK: Regex = Regex::new(r"\.bak(\d)?$").unwrap();
+}
+
 trait ExplainValue {
     fn get_len(&self) -> Option<u64>;
     fn get_type(&self) -> String;
@@ -1058,7 +1078,11 @@ impl Database {
                 Ok(v) => {
                     let k = v.to_str().unwrap();
                     let key_name = &k[path_len + 1..k.len() - suffix_len];
-                    if hidden || !key_name.starts_with('.') {
+                    if hidden
+                        || (!key_name.starts_with('.')
+                            && !key_name.contains("/.")
+                            && !RE_BAK.is_match(key_name))
+                    {
                         result.push(key_name.to_owned());
                     }
                 }
@@ -1447,13 +1471,46 @@ impl Database {
 
     pub fn key_get_recursive(&mut self, key: &str) -> Result<Vec<(String, Value)>, Error> {
         let mut result = Vec::new();
-        for key in self.list_key_and_subkeys(key, true)? {
+        for key in self.list_key_and_subkeys(key, false)? {
             let value = self.get_key_data(DataKey::Name(&key), false)?.0;
             result.push((key, value));
         }
         Ok(result)
     }
 
+    pub fn key_get_field(&mut self, key: &str, field: &str) -> Result<Value, Error> {
+        let key_data = self.key_get(key)?;
+        match key_data {
+            Value::Object(o) => match o.get(field) {
+                Some(v) => Ok(v.clone()),
+                None => Err(Error::new(
+                    ErrorKind::FieldNotFound,
+                    format!("no such field: {}", field),
+                )),
+            },
+            _ => Err(Error::new(ErrorKind::DataError, "key is not object")),
+        }
+    }
+
+    pub fn key_set_field(&mut self, key: &str, field: &str, value: Value) -> Result<(), Error> {
+        let key_data = match self.key_get(key) {
+            Ok(v) => v,
+            Err(e) if e.kind() == ErrorKind::KeyNotFound => {
+                let m = serde_json::map::Map::new();
+                Value::from(m)
+            }
+            Err(e) => return Err(e),
+        };
+        match key_data {
+            Value::Object(mut o) => {
+                o.insert(field.to_owned(), value);
+                self.key_set(key, Value::from(o))?;
+                Ok(())
+            }
+            _ => Err(Error::new(ErrorKind::DataError, "key is not object")),
+        }
+    }
+
     pub fn key_set(&mut self, key: &str, value: Value) -> Result<(), Error> {
         if self.auto_bak > 0 {
             for n in (1..self.auto_bak + 1).rev() {
